--[[
Author: Starinnia
CPR is a combo points display addon based on Funkydude's BasicComboPoints
$Date: 2009-01-09 02:21:52 +0000 (Fri, 09 Jan 2009) $
$Revision: 26 $

Copyright (c) 2007-2008 Michael J. Murray aka Starinnia of Lothar

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
]]

local basepath = "Interface\\Addons\\ComboPointsRedux\\Icons\\"
local GetComboPoints = GetComboPoints
local graphics = nil
local text = nil
local db
local defaults = {
	profile = {
		textpos = {},
		graphicpos = {},
		locked = false,
		font = "Friz Quadrata TT",
		graphicscale = 1,
		fontsize = 12,
		spacing = 5,
		normColorr = 1,
		normColorg = 0.9,
		normColorb = 0,
		fullColorr = 1,
		fullColorg = 0,
		fullColorb = 0,
		textColorr = 1,
		textColorg = 1,
		textColorb = 1,
		alpha = 1,
		icon = "square",
		orientation = "h",
	}
}
	
local ComboPointsRedux = LibStub("AceAddon-3.0"):NewAddon("ComboPointsRedux", "AceConsole-3.0", "AceEvent-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ComboPointsRedux")

local cprOps
local function getCPROps()
	if not cprOps then
		cprOps = {
			type = 'group',
			childGroups = 'tab',
			name = "ComboPointsRedux",
			args = {
				main = {
					type = 'group',
					name = L["Options"],
					args = {
						lock = {
							type = "toggle",
							name = L["Lock"],
							get = function() return db.locked end,
							set = function()
								if not db.locked then
									graphics:SetBackdropColor(1,1,1,0)
									text:SetBackdropColor(1,1,1,0)
									graphics:EnableMouse(false)
									graphics:SetMovable(false)
									text:EnableMouse(false)
									text:SetMovable(false)
									db.locked = true
								else
									graphics:SetBackdropColor(1,1,1,1)
									text:SetBackdropColor(1,1,1,1)
									graphics:EnableMouse(true)
									graphics:SetMovable(true)
									text:EnableMouse(true)
									text:SetMovable(true)
									db.locked = false
								end
							end,
							order = 1,
						},
						orientation = {
							type = 'select',
							name = L["Orientation"],
							get = function() return db.orientation end,
							set = function(_, value)
								db.orientation = value
								for i = 1, 5 do
									graphics.points[i]:ClearAllPoints()
								end
								if value == "v" then
									graphics.points[1]:SetPoint("BOTTOM", graphics, "BOTTOM", 0, 0)
									graphics.points[2]:SetPoint("BOTTOM", graphics.points[1], "TOP", 0, db.spacing*db.graphicscale)
									graphics.points[3]:SetPoint("BOTTOM", graphics.points[2], "TOP", 0, db.spacing*db.graphicscale)
									graphics.points[4]:SetPoint("BOTTOM", graphics.points[3], "TOP", 0, db.spacing*db.graphicscale)
									graphics.points[5]:SetPoint("BOTTOM", graphics.points[4], "TOP", 0, db.spacing*db.graphicscale)
									graphics:SetHeight(145*db.graphicscale)
									graphics:SetWidth(25*db.graphicscale)
								else
									graphics.points[1]:SetPoint("LEFT", graphics, "LEFT", 0, 0)
									graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", db.spacing*db.graphicscale, 0)
									graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", db.spacing*db.graphicscale, 0)
									graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", db.spacing*db.graphicscale, 0)
									graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", db.spacing*db.graphicscale, 0)
									graphics:SetWidth(145*db.graphicscale)
									graphics:SetHeight(25*db.graphicscale)
								end
							end,
							values = { h = L["Horizontal"], v = L["Vertical"], },
							order = 2,
						},
						font = {
							type = "select",
							name = L["Font"],
							get = function() return db.font end,
							set = function(_, v)
								db.font = v
								text.t:SetFont(LSM:Fetch('font', v), db.fontsize)
							end,
							dialogControl = "LSM30_Font",
							values = AceGUIWidgetLSMlists.font,
							order = 3,
						},
						fontsize = {
							type = "range",
							name = L["Font Size"],
							get = function() return db.fontsize end,
							set = function(_, v)
								db.fontsize = v
								text.t:SetFont(LSM:Fetch('font', db.font), v)
							end,
							min = 1,
							max = 36,
							step = 1,
							width = 'full',
							order = 4,
						},
						graphicscale = {
							type = "range",
							name = L["Graphics Scale"],
							get = function() return db.graphicscale end,
							set = function(_, scale)
								db.graphicscale = scale
								for i = 1, 5 do
									graphics.points[i]:SetHeight(25*scale)
									graphics.points[i]:SetWidth(25*scale)
								end
								graphics:SetHeight(25*scale)
								graphics:SetWidth(145*scale)
								graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", db.spacing*scale, 0)
								graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", db.spacing*scale, 0)
								graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", db.spacing*scale, 0)
								graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", db.spacing*scale, 0)
							end,
							min = .1,
							max = 1.5,
							step = .01,
							width = 'full',
							order = 6,
						},
						spacing = {
							type = 'range',
							name = L["Spacing"],
							get = function() return db.spacing end,
							set = function(_, v)
								db.spacing = v
								graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", v*db.graphicscale, 0)
								graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", v*db.graphicscale, 0)
								graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", v*db.graphicscale, 0)
								graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", v*db.graphicscale, 0)
							end,
							min = -2,
							max = 10,
							step = 1,
							width = 'full',
							order = 7,
						},
						alpha = {
							type = 'range',
							name = L["Alpha"],
							get = function() return db.alpha end,
							set = function(_, v)
								db.alpha = v
								text.t:SetAlpha(v)
								text.t:SetShadowColor(0, 0, 0, db.alpha)
								for i = 1, 5 do
									graphics.points[i]:SetAlpha(v)
								end
							end,
							min = 0,
							max = 1,
							step = .01,
							width = 'full',
							order = 8,
						},
						normalcolor = {
							type = 'color',
							name = L["Normal Color"],
							get = function() return db.normColorr, db.normColorg, db.normColorb end,
							set = function(_, r, g, b)
								db.db.normColorr = r
								db.db.normColorg = g
								db.normColorb = b
								ComboPointsRedux:ReDraw()
							end,
							order = 9,
						},
						fullcolor = {
							type = 'color',
							name = L["Five Point Color"],
							get = function() return db.fullColorr, db.fullColorg, db.fullColorb end,
							set = function(_, r, g, b)
								db.fullColorr = r
								db.fullColorg = g
								db.fullColorb = b
								ComboPointsRedux:ReDraw()
							end,
							order = 10,
						},
						textcolor = {
							type = 'color',
							name = L["Text Color"],
							get = function() return db.textColorr, db.textColorg, db.textColorb end,
							set = function(_, r, g, b)
								db.textColorr = r
								db.textColorg = g
								db.textColorb = b
								text.t:SetTextColor(r, g, b)
							end,
							order = 11,
						},
						icon = {
							type = 'select',
							name = L["Combo Point Icon"],
							get = function() return db.icon end,
							set = function(_, v)
								db.icon = v
								for i = 1, 5 do
									graphics.points[i].icon:SetTexture(basepath..db.icon)
								end
							end,
							values = { square = L["Square"], circle = L["Circle"], triangle = L["Triangle"], diamond = L["Diamond"], },
							order = 12,
						},
					},
				},
				profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(ComboPointsRedux.db)
			},
		}
	end
	return cprOps
end

function ComboPointsRedux:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("CPRDB", defaults, "Default")
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ComboPointsRedux", getCPROps)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ComboPointsRedux", "ComboPointsRedux")
	self:RegisterChatCommand("cpr", "OpenConfig", true, true)
end

function ComboPointsRedux:OnEnable()
	self:MakeTextFrame()
	self:MakeGraphicsFrame()
	
	self:RegisterEvent("UNIT_COMBO_POINTS", "ReDraw")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "ReDraw")
end

function ComboPointsRedux:Reset()
	text:ClearAllPoints()
	graphics:ClearAllPoints()
	
	text:SetPoint("CENTER", UIParent, "CENTER")
	graphics:SetPoint("CENTER", UIParent, "CENTER")
	
	for i = 1, 5 do
		graphics.points[i].icon:SetTexture(basepath.."square")
	end
	
	graphics:SetWidth(145)
	graphics:SetHeight(25)
end

function ComboPointsRedux:Refresh()
	text:ClearAllPoints()
	graphics:ClearAllPoints()
	
	if db.textpos.x then
		text:SetPoint(db.textpos.point, UIParent, db.textpos.anchor, db.textpos.x, db.textpos.y)
		graphics:SetPoint(db.graphicpos.point, UIParent, db.graphicpos.anchor, db.graphicpos.x, db.graphicpos.y)
	else
		text:SetPoint("CENTER", UIParent, "CENTER")
		graphics:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	for i = 1, 5 do
		graphics.points[i].icon:SetTexture(basepath..db.icon)
	end
	
	if db.orientation == "v" then
		graphics.points[1]:SetPoint("BOTTOM", graphics, "BOTTOM", 0, 0)
		graphics.points[2]:SetPoint("BOTTOM", graphics.points[1], "TOP", 0, db.spacing*db.graphicscale)
		graphics.points[3]:SetPoint("BOTTOM", graphics.points[2], "TOP", 0, db.spacing*db.graphicscale)
		graphics.points[4]:SetPoint("BOTTOM", graphics.points[3], "TOP", 0, db.spacing*db.graphicscale)
		graphics.points[5]:SetPoint("BOTTOM", graphics.points[4], "TOP", 0, db.spacing*db.graphicscale)
		graphics:SetHeight(145*db.graphicscale)
		graphics:SetWidth(25*db.graphicscale)
	else
		graphics.points[1]:SetPoint("LEFT", graphics, "LEFT", 0, 0)
		graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", db.spacing*db.graphicscale, 0)
		graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", db.spacing*db.graphicscale, 0)
		graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", db.spacing*db.graphicscale, 0)
		graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", db.spacing*db.graphicscale, 0)
		graphics:SetWidth(145*db.graphicscale)
		graphics:SetHeight(25*db.graphicscale)
	end
	
	if not db.locked then
		text:SetBackdropColor(1,1,1,0)
		text:EnableMouse(false)
		text:SetMovable(false)
		graphics:SetBackdropColor(1,1,1,0)
		graphics:EnableMouse(false)
		graphics:SetMovable(false)
	end
end

function ComboPointsRedux:OpenConfig()
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("ComboPointsRedux", 500, 550)
	LibStub("AceConfigDialog-3.0"):Open("ComboPointsRedux")
end

function ComboPointsRedux:ReDraw()
	--see if the player has a vehicle, if they do get the vehicle's CPs
	local points = GetComboPoints(UnitHasVehicleUI("player") and "vehicle" or "player", "target")
	
	if points == 0 then
		for i = 1, 5 do
			graphics.points[i]:Hide()
			graphics.points[i].icon:SetVertexColor(db.normColorr, db.normColorg, db.normColorb)
		end
		points = ""
	elseif points == 5 then
		for i = points, 1, -1 do
			graphics.points[i].icon:SetVertexColor(db.fullColorr, db.fullColorg, db.fullColorb)
			graphics.points[i]:Show()
		end
	else
		for i = points, 1, -1 do
			graphics.points[i]:Show()
		end
	end
	
	text.t:SetText(points)
end

local bg = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",}
function ComboPointsRedux:MakeTextFrame()
	text = CreateFrame("FRAME", "CPRTEXTFRAME", UIParent)
	text:SetBackdrop(bg)
	text:SetFrameStrata("BACKGROUND")
	text:SetClampedToScreen(true)
	if db.textpos.x then
		text:SetPoint(db.textpos.point, UIParent, db.textpos.anchor, db.textpos.x, db.textpos.y)
	else
		text:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
	text:SetBackdropColor(1,1,1,0)
	text:SetWidth(50)
	text:SetHeight(50)
	
	text.t = text:CreateFontString(nil,"OVERLAY")
	text.t:SetAllPoints(text)
	text.t:SetShadowColor(0, 0, 0, db.alpha)
	text.t:SetShadowOffset(1, -1)
	text.t:SetFont(LSM:Fetch("font", db.font), db.fontsize)
	text.t:SetTextColor(db.textColorr, db.textColorg, db.textColorb)
	text.t:SetAlpha(db.alpha)
	
	if not db.locked then
		text:SetBackdropColor(1,1,1,1)
		text:EnableMouse(true)
		text:SetMovable(true)
	end
	
	text:RegisterForDrag("LeftButton")
	text:SetScript("OnDragStart", function() this:StartMoving() end)
	text:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		local point, _, anchor, x, y = this:GetPoint()
		db.textpos.x = math.floor(x)
		db.textpos.y = math.floor(y)
		db.textpos.anchor = anchor
		db.textpos.point = point
	end)
	
	text:Show()
end

function ComboPointsRedux:MakeGraphicsFrame()
	graphics = CreateFrame("FRAME", "CPRGRAPHICSFRAME", UIParent)
	graphics:SetBackdrop(bg)
	graphics:SetFrameStrata("BACKGROUND")
	graphics:SetClampedToScreen(true)
	if db.graphicpos.x then
		graphics:SetPoint(db.graphicpos.point, UIParent, db.graphicpos.anchor, db.graphicpos.x, db.graphicpos.y)
	else
		graphics:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end
	graphics:SetBackdropColor(1,1,1,0)
	if db.orientation == "v" then
		graphics:SetHeight(145*db.graphicscale)
		graphics:SetWidth(25*db.graphicscale)
	else
		graphics:SetWidth(145*db.graphicscale)
		graphics:SetHeight(25*db.graphicscale)
	end
	
	if not db.locked then
		graphics:SetBackdropColor(1,1,1,1)
		graphics:EnableMouse(true)
		graphics:SetMovable(true)
	end
	
	graphics:RegisterForDrag("LeftButton")
	graphics:SetScript("OnDragStart", function() this:StartMoving() end)
	graphics:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		local point, _, anchor, x, y = this:GetPoint()
		db.graphicpos.x = math.floor(x)
		db.graphicpos.y = math.floor(y)
		db.graphicpos.anchor = anchor
		db.graphicpos.point = point
	end)
	
	graphics.points = {}
	for i = 1, 5 do
		graphics.points[i] = CreateFrame("FRAME", nil, graphics)
		graphics.points[i].icon = graphics.points[i]:CreateTexture(nil, "OVERLAY")
		graphics.points[i].icon:SetAllPoints(graphics.points[i])
		graphics.points[i].icon:SetTexture(basepath..db.icon)
		graphics.points[i].icon:SetVertexColor(db.normColorr, db.normColorg, db.normColorb)
		graphics.points[i]:SetAlpha(db.alpha)
		graphics.points[i]:SetHeight(25*db.graphicscale)
		graphics.points[i]:SetWidth(25*db.graphicscale)
		graphics.points[i]:Hide()
	end
	
	if db.orientation == "v" then
		graphics.points[1]:SetPoint("BOTTOM", graphics, "BOTTOM", 0, 0)
		graphics.points[2]:SetPoint("BOTTOM", graphics.points[1], "TOP", 0, db.spacing*db.graphicscale)
		graphics.points[3]:SetPoint("BOTTOM", graphics.points[2], "TOP", 0, db.spacing*db.graphicscale)
		graphics.points[4]:SetPoint("BOTTOM", graphics.points[3], "TOP", 0, db.spacing*db.graphicscale)
		graphics.points[5]:SetPoint("BOTTOM", graphics.points[4], "TOP", 0, db.spacing*db.graphicscale)
	else
		graphics.points[1]:SetPoint("LEFT", graphics, "LEFT", 0, 0)
		graphics.points[2]:SetPoint("LEFT", graphics.points[1], "RIGHT", db.spacing*db.graphicscale, 0)
		graphics.points[3]:SetPoint("LEFT", graphics.points[2], "RIGHT", db.spacing*db.graphicscale, 0)
		graphics.points[4]:SetPoint("LEFT", graphics.points[3], "RIGHT", db.spacing*db.graphicscale, 0)
		graphics.points[5]:SetPoint("LEFT", graphics.points[4], "RIGHT", db.spacing*db.graphicscale, 0)
	end
	
	graphics:Show()
end
